package ShowXPath.GUI;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.tree.DefaultMutableTreeNode;
import ShowXPath.TreeNode.*;
import ShowXPath.StackNode.*;
import ShowXPath.XMLParser.*;
import ShowXPath.*;


class XMLNodeSelected implements TreeSelectionListener
{
	Main _main;
	public XMLNodeSelected(Main main)
	{
		_main = main;
	}

	private void print(String string)
	{
		//System.out.println("In XMLTree: " + string);
	}
	
	public void valueChanged(TreeSelectionEvent e)
	{
		TreePath t = e.getNewLeadSelectionPath();
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode) t.getLastPathComponent();
		print("Selected: " + node.toString());
		if (_main != null)
		{
			_main.setContext((LTreeNode)node.getUserObject());
		}
	}
}

public class XMLTree extends javax.swing.JPanel
{

	private static Icon iconRoot = new ImageIcon("images/root.gif");
	private static Icon iconElement = new ImageIcon("images/element.gif");
	private static Icon iconAttribute = new ImageIcon("images/attribute.gif");
	private static Icon iconProcessingInstruction
	     = new ImageIcon("images/pi.gif");
	private static Icon iconComment = new ImageIcon("images/comment.gif");
	private static Icon iconText = new ImageIcon("images/text.gif");
	private static Icon iconNamespace = new ImageIcon("images/namespace.gif");

	Main _main;
	LStackNode _node = new LNodeset();
	JTree tree = new javax.swing.JTree();

	private JTree getTree()
	{
		return tree;
	}

	public void showNodes(LStackNode node)
	{
		if (node instanceof LNodeset)
		{
			_node = node;
			this.invalidate();
			this.validate();
			this.repaint();
		}
	}
	
  private class CellRenderer
      implements TreeCellRenderer {


    private JLabel renderer;

    CellRenderer () {
      renderer = new JLabel();
      renderer.setOpaque (true);
    }

    public Component  getTreeCellRendererComponent(JTree tree,
        Object value, boolean selected, boolean expanded,
        boolean leaf, int row, boolean hasFocus) {

			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
			
			LTreeNode treeNode = (LTreeNode)node.getUserObject();
			String string = treeNode.toString();

			renderer.setBackground((selected) ? Color.lightGray : Color.white);

			if (((LNodeset)_node).contains(treeNode))
			{
				renderer.setForeground(Color.red);
				/*
				TreeNode tn[] = node.getPath();
				TreePath tp = new TreePath(node.getPath());
				getTree().expandPath(tp);
				getTree().scrollPathToVisible(tp);
				*/
			}
			else
			{
				renderer.setForeground(Color.black);
			}
/*			
			String htmlPrefix = "<html><font color=blue>";
			String htmlNormal = "</font><font color=black><xmp>";
			String htmlSelected = "</font><font color=red><xmp>";
			String htmlLast = "</xmp></font></html>";
*/
			String htmlPrefix = "";
			String htmlNormal = "";
			String htmlSelected = "";
			String htmlLast = "";

			String type = "";
			Icon icon;
			
			if (treeNode instanceof LRoot)
			{
				type = "Root:";
				icon = iconRoot;
			}
			else if (treeNode instanceof LElement)
			{
				type = "Element:";
				icon = iconElement;
			}
			else if (treeNode instanceof LAttribute)
			{
				type = "Attribute:";
				icon = iconAttribute;
			}
			else if (treeNode instanceof LComment)
			{
				type = "Comment:";
				icon = iconComment;
			}
			else if (treeNode instanceof LText)
			{
				type = "Text:";
				icon = iconText;
			}
			else if (treeNode instanceof LProcessingInstruction)
			{
				type = "Processing Instruction:";
				icon = iconProcessingInstruction;
			}
			else if (treeNode instanceof LNamespace)
			{
				type = "Namespace:";
				icon = iconNamespace;
			}
			else
			{
				icon = iconElement;
				type = "None: ";
			}
			type = "[" + treeNode.getID() + "]: ";
			//type = type + "[" + treeNode.getID() + "]: ";
			String theString;
			if (((LNodeset)_node).contains(treeNode))
			{
				theString = htmlPrefix + type + htmlSelected + string + htmlLast;
			}
			else
			{
				theString = htmlPrefix + type + htmlNormal + string + htmlLast;
			}
			renderer.setText(theString);
			renderer.setIcon(icon);
			renderer.setBackground((selected) ? Color.pink : Color.white);
			return renderer;
    }
  }

	private static void duplicateStructure(LTreeNode root,
																				 DefaultMutableTreeNode treeNode)
	{
		LNodeset ns = root.namespace();
		ListIterator k = ns.listIterator();

		while (k.hasNext())
		{
			LNamespace name = (LNamespace) k.next();
					
			DefaultMutableTreeNode tnode =
				new DefaultMutableTreeNode(name, false);
			try
			{
				treeNode.add(tnode);
			}
			catch (java.lang.IllegalArgumentException iae)
			{
			}
			catch (java.lang.IllegalStateException ise)
			{
			}
		}

		LNodeset a = root.attribute();
		ListIterator j = a.listIterator();

		while (j.hasNext())
		{
			LAttribute attr = (LAttribute) j.next();
					
			DefaultMutableTreeNode tnode =
				new DefaultMutableTreeNode(attr, false);
			try
			{
				treeNode.add(tnode);
			}
			catch (java.lang.IllegalArgumentException iae)
			{
			}
			catch (java.lang.IllegalStateException ise)
			{
			}
		}

		LNodeset children;
		if (root instanceof LRoot)
		{
			children = ((LRoot)root).getChildren();
		}
		else if (root instanceof LElement)
		{
			children = ((LElement)root).getChildren();
		}
		else 
		{
			children = root.child();
		}
		
		ListIterator i = children.listIterator();

		while (i.hasNext())
		{
			LTreeNode n = (LTreeNode) i.next();
			DefaultMutableTreeNode t = new DefaultMutableTreeNode(n);
			try
			{
				treeNode.add(t);
			}
			catch (java.lang.IllegalArgumentException iae)
			{
			}
			catch (java.lang.IllegalStateException ise)
			{
			}

			if (n instanceof LElement)
			{
				duplicateStructure(n, t);
			}
		}        
	}

	public void addTreeSelectionListener(Object obj)
	{
		tree.addTreeSelectionListener(
			(javax.swing.event.TreeSelectionListener)obj
			);
	}

	public java.lang.Object getLastSelectedPathComponent()
	{
		return tree.getLastSelectedPathComponent();
	}

	public XMLTree(LRoot root, Main main)
	{
		DefaultMutableTreeNode treeRoot;

		_main = main;

		treeRoot = new DefaultMutableTreeNode(root);
		duplicateStructure(root, treeRoot);
		tree = new JTree(treeRoot);
		tree.setCellRenderer(new CellRenderer());
		for (Enumeration e = treeRoot.depthFirstEnumeration(); e.hasMoreElements();)
		{
			DefaultMutableTreeNode n = (DefaultMutableTreeNode) e.nextElement();
			tree.expandPath(new TreePath(n.getPath()));
		}


		setLayout(new BorderLayout(0,0));
		setBackground(java.awt.Color.white);
		add("Center", tree);
		tree.setBounds(0,0,700,500);
		tree.getSelectionModel().setSelectionMode(
			TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.putClientProperty("JTree.lineStyle", "Angled");
		XMLNodeSelected selectionListener = new XMLNodeSelected(_main);
		
		tree.addTreeSelectionListener(selectionListener);
	}

	public XMLTree(LRoot root)
	{
		DefaultMutableTreeNode treeRoot;

		_main = null;

		treeRoot = new DefaultMutableTreeNode(root);
		duplicateStructure(root, treeRoot);

		tree = new JTree(treeRoot);
		tree.setCellRenderer(new CellRenderer());
		for (Enumeration e = treeRoot.depthFirstEnumeration(); e.hasMoreElements();)
		{
			DefaultMutableTreeNode n = (DefaultMutableTreeNode) e.nextElement();
			tree.expandPath(new TreePath(n.getPath()));
		}

		setLayout(new BorderLayout(0,0));
		setBackground(java.awt.Color.white);
		add("Center", tree);
		tree.setBounds(0,0,700,500);
		tree.getSelectionModel().setSelectionMode(
			TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.putClientProperty("JTree.lineStyle", "Angled");
		XMLNodeSelected selectionListener = new XMLNodeSelected(_main);
		
		tree.addTreeSelectionListener(selectionListener);
	}
}
