package ShowXPath.TreeNode;

import java.util.*;
import java.io.*;

import ShowXPath.StackNode.*;

class ExternalEntity
{
	private String _name;
	private String _systemID;
	private String _publicID;
	private String _notation;

	public ExternalEntity(String name, String publicId, String systemId,
												String notation)
	{
		_name = name;
		_systemID = systemId;
		_publicID = publicId;
		_notation = notation;
	}

	public String getURI()
	{
		return _systemID;
	}
	
}

class AttributeDTD
{
	private String _eName;
	private String _aName;
	private String _type;
	private String _valueDefault;
	private String _value;

	public AttributeDTD(String eName, String aName, String type,
											String valueDefault, String value)
	{
		_eName = eName;
		_aName = aName;
		_type = type;
		_valueDefault = valueDefault;
		_value = value;
	}

	public String getType()
	{
		return _type;
	}
	
}

class ElementDTD
{
	private String _name;
	private String _model;
	private Hashtable _attributes = new Hashtable();

	public ElementDTD(String name, String model)
	{
		_name = name;
		_model = model;
	}

	public AttributeDTD getAttribute(String name)
	{
		return (AttributeDTD) _attributes.get(name);
	}
	
	public void attributeDecl (String eName,
														 String aName,
														 String type,
														 String valueDefault,
														 String value)
	{
		AttributeDTD ad = new AttributeDTD(eName, aName, type, valueDefault,
																			 value);
		_attributes.put(aName, ad);
	}
}

class DocumentDTD
{
	private String _name;
	private String _publicID;
	private String _systemID;
	private Hashtable _elements = new Hashtable();

	public ElementDTD getElement(String name)
	{
		return (ElementDTD)_elements.get(name);
	}

	public void startDTD(String name, String publicID, String systemID)
	{
		_name = name;
		_publicID = publicID;
		_systemID = systemID;
	}

	public void elementDecl(String name, String model)
	{
		ElementDTD ed = new ElementDTD(name, model);
		_elements.put(name, ed);
	}

	public void attributeDecl (String eName,
														 String aName,
														 String type,
														 String valueDefault,
														 String value)
	{
		ElementDTD ed = (ElementDTD) _elements.get(eName);
		if (ed != null)
		{
			ed.attributeDecl(eName, aName, type, valueDefault, value);
		}
	}
}



public class LRoot extends LTreeNode
{
	// TODO
	//   1. The URI for the document.
	private String _encoding;
	private LNodeset _children = new LNodeset();
	private static String _type = "LRoot";
	private Hashtable _unparsedEntities = new Hashtable();
	private DocumentDTD _documentDTD = new DocumentDTD();
	private Hashtable _elementIDs = new Hashtable();

	public LRoot getRoot()
	{
		return this;
	}

	private void print(String string)
	{
		//System.out.println("in LRoot: " + string);
	}

	public LRoot()
	{
		setID();
	}

	public void setElementID(String id, LElement element)
	{
		_elementIDs.put(id, element);
	}

	public boolean isIDAttribute(String eName, String aName)
	{
		print("is IDAttribute. eName = " + eName + " aName = " + aName);
		ElementDTD element = (ElementDTD) _documentDTD.getElement(eName);
		if (element != null)
		{
			print("element is no null");
			AttributeDTD attribute = element.getAttribute(aName);
			if (attribute == null)
			{
				print("Attribute is null");
				return false;
			}
			else
			{
				if (attribute.getType().equals("ID"))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
		}
		else
		{
			return false;
		}
	}

	public LElement getElementID(String id)
	{
		return (LElement) _elementIDs.get(id);
	}

	public void addUnparsedEntity(String name, String URI, String publicID,
																String notation)
	{
		ExternalEntity ee = new ExternalEntity(name, publicID, URI, notation);
		_unparsedEntities.put(name, ee);
	}

	public String getUnparsedEntityURI(String name)
	{
		return (String) _unparsedEntities.get(name);
	}

	public String getLang()
	{
		return "";
	}
	
	public LTreeNode getParent()
	{
		return null;
	}

	public LNodeset getChildren()
	{
		return _children;
	}

	public void addChild(LTreeNode node)
	{
		_children.addLast(node);
	}

	public String getStringValue()
	{
		LNodeset nodes = descendent();
		Iterator i = nodes.listIterator();
		String s = "";
		while (i.hasNext())
		{
			Object o = i.next();
			if (o instanceof LText)
			{
				s = s + ((LText)o).getStringValue();
			}
		}
		return s;
	}

	public String getName()
	{
		return "";
	}

	public String getLocal_name()
	{
		return "";
	}

	public String getNamespace()
	{
		return "";
	}

	public String getNamespace_uri()
	{
		return "";
	}

	public String toString()
	{
		return "<?xml version='1.0'?>";
	}
	
	public String getNodeValue()
	{
		String s;
		s = "<?xml {" + _order + "}" + "version='1.0'?>\n";
		ListIterator i = _children.listIterator();
		while (i.hasNext())
		{
			s = s + ((LTreeNode)i.next()).toString();
		}
		return s;
	}

	public LNodeset parent()
	{
		LNodeset list = new LNodeset();
		return list;
	}

	public LNodeset child()
	{
		LNodeset list = new LNodeset();
		list.addAll(_children);
		return list;
	}

	public LNodeset ancestor()
	{
		LNodeset list = new LNodeset();
		return list;
	}

	public LNodeset self()
	{
		LNodeset list = new LNodeset();
		list.add(this);
		return list;
	}

	public LNodeset ancestor_or_self()
	{
		LNodeset list = new LNodeset();
		list.add(this);
		return list;
	}

	public LNodeset descendent()
	{
		LNodeset list = new LNodeset();
		list.addAll(_children);
		ListIterator i = _children.listIterator();
		while (i.hasNext())
		{
			list.addAll(((LTreeNode)i.next()).descendent());
		}
		return list;
	}

	public LNodeset descendent_or_self()
	{
		LNodeset list = descendent();
		list.add(this);
		return list;
	}

	public LNodeset attribute()
	{
		LNodeset list = new LNodeset();
		return list;
	}

	public LNodeset following()
	{
		return descendent();
	}

	public LNodeset following_sibling()
	{
		LNodeset list = new LNodeset();
		return list;
	}

	public LNodeset preceding()
	{
		LNodeset list = new LNodeset();
		return list;
	}

	public LNodeset preceding_sibling()
	{
		LNodeset list = new LNodeset();
		return list;
	}

	public LNodeset namespace()
	{
		LNodeset list = new LNodeset();
		return list;
	}

	public String showNode()
	{
		return "<?xml {" + _order + "}" + "version='1.0'?>\n";
	}

	public LBoolean toLBoolean()
	{
		return null;
	}

	public LNumber toLNumber()
	{
		return null;
	}

	public LString toLString()
	{
		return null;
	}

	public String type()
	{
		return _type;
	}

	public void startDTD(String name, String publicID, String systemID)
	{
		_documentDTD.startDTD(name, publicID, systemID);
	}

	public void elementDecl(String name, String model)
	{
		_documentDTD.elementDecl(name, model);
	}

	public void attributeDecl (String eName,
														 String aName,
														 String type,
														 String valueDefault,
														 String value)
	{
		_documentDTD.attributeDecl(eName, aName, type, valueDefault, value);
	}
	
}

